\name{RunModel}
\alias{RunModel}
\title{
Run the mcmc sampler for the model.
}
\description{
Run the mcmc sampler for the model.
}
\usage{
RunModel(orig,mc,hyper,para,output,synindex,individual_variable_index,
    household_variable_index,HHhead_at_group_level,weight_option,struc_weight,MissData)
}
\arguments{
  \item{orig}{A list object including all the necessary data variables needed by the sampler.; output of the \verb{initData} function.}
  \item{mc}{A list specifying the number of mcmc iterations, burn-in, thinning and the effective sample size.}
  \item{hyper}{Hyper parameters for the prior distributions.}
  \item{para}{A list of the initial values of the parameters; output of the \verb{initParameters} function.}
  \item{output}{A list of output parameters to be saved; output of the \verb{initOutput} function.}
  \item{synindex}{A vector of iteration indexes for sampling synthetic data. length(\verb{synindex}) is the number of synthetic data needed. }
  \item{individual_variable_index}{Vector of column indexes for the individual-level variables.}
  \item{household_variable_index}{Vector of column indexes for the group-level variables.}
  \item{HHhead_at_group_level}{Logical indicator for whether or not to move the household head to the household level. Set to TRUE to move the household head and FALSE otherwise.}
  \item{weight_option}{Logical indicator for whether or not to cap the number of impossible households to sample and re-weight the multinomial counts within each latent class back to the expected truth. Set to TRUE to use the weigthting option nd FALSE otherwise.}
  \item{struc_weight}{Vector specifying the weights to be used for each household size. The weights must be ordered by household sizes and no household must be excluded.}
  \item{MissData}{A list that stores all the info related to missing data. Default to NULL for no missing data.}
}

\details{
This function runs the mcmc sampler for the NDPMPM model and generates posterior samples of parameters. It also generates synthetic data when needed.

Please note that:
\itemize{
\item{The minimum household size for this mcmc sampler is 2 because households of size 1 do not violate the structural zeros specified in this package. Also, moving the household head to the household level is not possible for households of size 1.}
\item{Each variable included must be recoded to start from 1.}
\item{Moving the household head to the household level and setting the HHhead_at_group_level option to TRUE speeds up the sampler significantly.}
\item{Setting the weight_option to TRUE and specifying weights also speeds up the sampler but the exact rate of speedup depends on the specific weights.}
}

Our example data set contains a sample of 2000 households and seven variables from the 2012 American Community Survey data. The variables are described below:
\itemize{
\item{ownership (ownership of dwelling): 1 = owned or being bought (loan), 2 = rented.}
\item{householdsize (household size): 2 = 2 people, 3 = 3 people, 4 = 4 people, 5 = 5 people, 6 = 6 people.}
\item{sex (gender): 1 = male, 2 = female.}
\item{race: 1 = white, 2 = black, 3 = American Indian or Alaska Native, 4 = Chinese, 5 = Japanese, 6 = other Asian/Pacific Islander,
7 = other race, 8 = two major races, 9 = three/more major races.}
\item{hisp (Hispanic origin). 1 = not Hispanic, 2 = Mexican, 3 = Puerto Rican, 4 = Cuban, 5 = other.}
\item{age: 1 = 0 (less then one year old), 2 = 1, 3 = 2,   . . . , 94 = 93}
\item{relate (relationship to the household head): 1 = head/householder, 2 = spouse, 3 = child, 4 = child-in-law, 5 = parent, 6 = parent-in-
law, 7 = sibling, 8 = sibling-in-law, 9 = grandchild, 10 = other relatives, 11 = partner, friend, visitor, 12 = other non-relatives}
}
Subsequent data sets must follow this structure because of the predefined list of structural zeros or users can modify the list of structural zeros by downloading the package source, making changes only to the checkconstraints_imp.cpp file and re-building the package.
}

\value{
\item{synData}{The list of synthetic data when the length(\verb{synindex}) > 0.}
\item{output}{The list of posterior samples for the parameters included in \verb{output}.}
}


\examples{
### We now show how the sampler works using a sample from the 2012 ACS.
### The sample consists of 2000 households of sizes 2 to 6.
### Empty environment and load required libraries
rm(list = ls())
library(NestedCategBayesImpute)
library(dplyr)


### Set indicator for whether of not to move the household head
### Also set indicator for the weighting/capping option
HHhead_at_group_level <- TRUE #move head to the group level
weight_option <- TRUE #use weighting/capping option


### Use data included in package; head has been moved to the group level
### prepare data and specify variable indexes
orig.file <- system.file("extdata","origdata_newFormat.txt",
              package="NestedCategBayesImpute")
orig.data <- read.table(orig.file,header = TRUE, sep = " ")
orig.data$relate <- orig.data$relate - 1L #recode relate to 11 levels
household.size <- as.data.frame(table(orig.data$Hhindex))
household.size[,1] <- as.numeric(household.size[,1])
names(household.size) <- c("Hhindex", 'householdsize')
household <- orig.data \%>\% inner_join(household.size)
individual_variable_index = c(3:7)
household_variable_index = c(8:13) #last column must be household size


### Initialize the input data structure
orig <- initData(household,individual_variable_index,household_variable_index)


### Check first few lines of data; data contains households of sizes 2 to 6
head(household)


### Supply weights; one for each household size
### Also,must be ordered & no household size must be excluded
if(weight_option){
  struc_weight <- c(1/2,1/2,1/3,1/3,1/3)
} else {
  struc_weight <-rep(1,length(orig$n_star_h)) #just a dummy vector of ones
}


### Set mcmc parameters
mc <- list(nrun = 10, burn = 5, thin = 1)
mc$eff.sam <- (mc$nrun-mc$burn)/mc$thin


### Set number of categories for each household level variable
dHH <- rep(0,length(household_variable_index))
for (i in 1:length(dHH)) {
  dHH[i] <- max(household[,household_variable_index[i]])
  if (i == length(dHH) & !HHhead_at_group_level) {
  #When leaving head within household, model assumes that the HH size starts from 2
    dHH[length(dHH)] <- dHH[length(dHH)] - 1
  }
}


### Set hyper parameters
#aa, ab, ba & bb are gamma hyperparameters for alpha and beta
#blocksize is the number of households to sample at once (batch sampling)
#FF is the max number of group-level latent classes
#SS is the max number of individual-level classes
hyper <- list(FF=20 , SS=15, aa=0.25, ab=0.25,
    ba=0.25,bb=0.25,dHH = dHH, blocksize = 5000)


### Initialize parameters and output
para <- initParameters(orig,hyper,HHhead_at_group_level)
output <- initOutput(orig,hyper,mc)


### Set number of synthetic data and the mcmc indexes for them
mm <- 5
synindex <- sort(sample(seq((mc$burn +1),mc$nrun,by=mc$thin),mm,replace=FALSE))


### Run model
ModelResults <- RunModel(orig,mc,hyper,para,output,synindex,
                individual_variable_index,household_variable_index,
                HHhead_at_group_level,weight_option,struc_weight)


### View first few lines of the first synthetic data.
head((ModelResults$synData)[[1]])


### Some posterior summaries and plots
library(coda)
names(ModelResults$output)
dim(ModelResults$output$alphaout)
alpha_output <- mcmc(ModelResults$output$alphaout)
plot(alpha_output)
summary(alpha_output)

dim(ModelResults$output$betaout)
beta_output <- mcmc(ModelResults$output$betaout)
plot(beta_output)
summary(beta_output)

}

\author{
Quanli Wang, Olanrewaju Akande
}

\keyword{mcmc}
\keyword{model}

